@@ -143,10 +143,10 @@ parseRulesBody :: Parser Rule
 parseRulesBody = do 
         x <- parseRule
         choice [ do comma lang
-                    xs <- sepEndBy1 parseRule (comma lang)
+                    xs <- parseRule `sepEndBy1` comma lang
                     return (matchAny (x:xs))
                , do semi lang
-                    xs <- many1 (semi lang >> parseRule)
+                    xs <- parseRule `sepEndBy1` semi lang
                     return (matchFirst (x:xs))
                ,    return x
                ]
@@ -355,7 +355,7 @@ parseCondPrim = choice
             ) <?> "list of regular expressions"
             ])
         , char '$' >> choice 
-             [ do backref <- natural lang
+             [ do backref <- read <$> many1 digit
                   return $ CondString (getBackref backref)
              , do varname <- identifier lang 
                   choice 
@@ -373,6 +373,8 @@ parseCondPrim = choice
                            return $ CondTime (getTimeVar "sampleage")
                       , do guard $ varname == "date"
                            return $ CondDate (getDateVar "date")
+                      , do guard $ varname == "desktop"
+                           return $ CondString (getVar "desktop")
                      ]
               ] <?> "variable"
         , do regex <- parseRegex <?> "regular expression"
@@ -431,13 +433,10 @@ parseRegex = fmap (flip RE.compile [] . T.pack) $ lexeme lang $ choice
 -- | Parses a day-of-time specification (hh:mm)
 parseTime :: Parser NominalDiffTime
 parseTime = fmap fromIntegral $ lexeme lang $ do
-               h <- digitToInt <$> digit
-               mh <- optionMaybe (digitToInt <$> digit)
+               hour <- read <$> many1 digit
                char ':'
-               m1 <- digitToInt <$> digit
-               m2 <- digitToInt <$> digit
-               let hour = maybe h ((10*h)+) mh
-               return $ (hour * 60 + m1 * 10 + m2) * 60
+               minute <- read <$> count 2 digit
+               return $ (hour * 60 + minute) * 60
 
 parseDate :: Parser UTCTime
 parseDate = lexeme lang $ do
@@ -480,7 +479,7 @@ parseTagPart :: Parser (Ctx -> Maybe Text)
 parseTagPart = do parts <- many1 (choice
                         [ do char '$'
                              (replaceForbidden . ) <$> choice
-                               [ do num <- natural lang
+                               [ do num <- read <$> many1 digit
                                     return $ getBackref num
                                , do varname <- many1 (letter <|> oneOf ".")
                                     return $ getVar varname
@@ -522,6 +521,8 @@ getVar "title"   ctx = do
 getVar "program" ctx = do
                 (_,_,p) <- cWindowInScope ctx
                 return p
+getVar "desktop" ctx = do
+                return $ cDesktop (tlData (cNow ctx))
 getVar v ctx = error $ "Unknown variable " ++ v
 
 getNumVar :: String -> CtxFun Integer
