@@ -6,12 +6,14 @@ module Stats (
     ReportResults(..),
     ActivityFilter(..),
     Filter(..),
+    Repeater(..),
     defaultFilter,
     defaultReportOptions,
     parseActivityMatcher,
     filterPredicate,
     prepareCalculations,
-    processReports,
+    processReport,
+    processRepeater,
     renderReport
     ) where
 
@@ -26,12 +28,15 @@ import Data.MyText (Text,pack,unpack)
 import Data.Function (on)
 import System.Locale (defaultTimeLocale)
 import Control.Applicative
-import Data.Strict ((:!:))
+import Data.Strict ((:!:), Pair(..))
+import qualified Data.Strict as Strict
 import Data.Traversable (sequenceA)
+import Control.Arrow
 
 import Data
 import Categorize
 import LeftFold
+import DumpFormat
 
 
 data Report = GeneralInfos
@@ -40,6 +45,7 @@ data Report = GeneralInfos
     | EachCategory
     | IntervalCategory Category
     | IntervalTag Activity
+    | DumpSamples
         deriving (Show, Eq)
 
 data Filter = Exclude ActivityMatcher | Only ActivityMatcher | GeneralCond String
@@ -51,6 +57,9 @@ data ActivityMatcher = MatchActivity Activity | MatchCategory Category
 data ActivityFilter = ExcludeActivity ActivityMatcher | OnlyActivity ActivityMatcher
         deriving (Show, Eq)
 
+data Repeater = ByDay | ByMonth | ByYear
+        deriving (Show, Eq)
+
 -- Supported report output formats: text, comma-separated values and
 -- tab-separated values
 data ReportFormat = RFText | RFCSV | RFTSV
@@ -78,7 +87,9 @@ data ReportResults =
         | ListOfTimePercValues String [(String, String, Double)]
         | PieChartOfTimePercValues  String [(String, String, Double)]
         | ListOfIntervals String [Interval]
-        | MultpleReportResults [ReportResults]
+        | MultipleReportResults [ReportResults]
+        | RepeatedReportResults String [(String, ReportResults)]
+        | DumpResult (TimeLog (CaptureData, TimeZone, ActivityData))
 
 
 filterPredicate :: [Filter] -> TimeLogEntry (Ctx, ActivityData) -> Bool
@@ -88,6 +99,9 @@ filterPredicate filters tl =
                 Only act     -> onlyTag act tl
                 GeneralCond s-> applyCond s (cTimeZone (fst (tlData tl))) tl) filters
 
+filterActivity :: [ActivityFilter] -> ActivityData -> ActivityData
+filterActivity fs = filter (applyActivityFilter fs)
+
 applyActivityFilter :: [ActivityFilter] -> Activity -> Bool
 applyActivityFilter fs act = all go fs
     where go (ExcludeActivity matcher) = not (matchActivityMatcher matcher act)
@@ -160,59 +174,88 @@ calcSums = LeftFold M.empty
                 let go' m act = M.insertWith' (+) act (fromInteger (tlRate tl)/1000) m
                 in foldl' go' m (snd (tlData tl))) id
 
-processReports :: ReportOptions -> Calculations -> [Report] ->  LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) [ReportResults]
-processReports opts c = sequenceA . map (processReport opts c)
+processRepeater :: Repeater -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults
+processRepeater ByDay rep =
+    filterElems (\(b :!: _) -> b) $
+    pure (RepeatedReportResults "Day" . map (\(d,rr) -> (showGregorian d, rr)) . M.toList) <*>
+    multiplex (utctDay . tlTime . Strict.snd) rep
+processRepeater ByMonth rep =
+    filterElems (\(b :!: _) -> b) $
+    pure (RepeatedReportResults "Month" . map (\((y,m),rr) -> (show y ++ "-" ++ show m, rr)) . M.toList) <*>
+    multiplex ((\(y,m,_) -> (y, m)). toGregorian . utctDay . tlTime . Strict.snd) rep
+processRepeater ByYear rep =
+    filterElems (\(b :!: _) -> b) $
+    pure (RepeatedReportResults "Year" . map (\(y,rr) -> (show y, rr)) . M.toList) <*>
+    multiplex ((\(y,_,_) -> y). toGregorian . utctDay . tlTime . Strict.snd) rep
 
-processReport :: ReportOptions -> Calculations ->  Report -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults
-processReport opts ~(Calculations {..}) GeneralInfos =
-   pure (\n ->
-    ListOfFields "General Information"
+processReport :: ReportOptions -> Report -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults
+processReport opts GeneralInfos =
+   pure (\n firstDate lastDate ttr tts ->
+    let timeDiff = diffUTCTime lastDate firstDate
+        fractionRec = realToFrac ttr / (realToFrac timeDiff) :: Double
+        fractionSel = realToFrac tts / (realToFrac timeDiff) :: Double
+        fractionSelRec = realToFrac tts / realToFrac ttr :: Double
+    in ListOfFields "General Information"
         [ ("FirstRecord", show firstDate)
         , ("LastRecord",  show lastDate)
         , ("Number of records", show n)
-        , ("Total time recorded",  showTimeDiff totalTimeRec)
-        , ("Total time selected",  showTimeDiff totalTimeSel)
+        , ("Total time recorded",  showTimeDiff opts ttr)
+        , ("Total time selected",  showTimeDiff opts tts)
         , ("Fraction of total time recorded", printf "%3.0f%%" (fractionRec * 100))
         , ("Fraction of total time selected", printf "%3.0f%%" (fractionSel * 100))
         , ("Fraction of recorded time selected", printf "%3.0f%%" (fractionSelRec * 100))
         ]) <*>
-    onAll lfLength
+    onAll lfLength <*>
+    onAll calcFirstDate <*>
+    onAll calcLastDate <*>
+    onAll calcTotalTime <*>
+    onSelected calcTotalTime
 
-processReport opts ~(Calculations {..}) TotalTime =
-        pure $ 
+processReport opts  TotalTime =
+    onSelected $
+        pure (\totalTimeSel sums -> 
             ListOfTimePercValues "Total time per tag" .
             mapMaybe (\(tag,time) ->
                   let perc = realToFrac time/realToFrac totalTimeSel
                       pick = applyActivityFilter (roActivityFilter opts) tag
                   in if pick && perc*100 >= roMinPercentage opts
                   then Just $ ( show tag
-                              , showTimeDiff time
+                              , showTimeDiff opts time
                               , perc)
                   else Nothing
                   ) .
             reverse .
             sortBy (comparing snd) $
             M.toList $
-            sums
+            sums) <*>
+    calcTotalTime <*>
+    calcSums 
 
-processReport opts c (Category cat) = pure (processCategoryReport opts c cat)
+processReport opts (Category cat) = pure (\c -> processCategoryReport opts c cat) <*>
+    prepareCalculations
 
-processReport opts c EachCategory = 
-    pure (\cats -> MultpleReportResults $ map (processCategoryReport opts c) cats) <*>
+processReport opts EachCategory = 
+    pure (\c cats -> MultipleReportResults $ map (processCategoryReport opts c) cats) <*>
+    prepareCalculations <*>
     onSelected calcCategories
 
-processReport opts c (IntervalCategory cat) =
-    processIntervalReport opts c ("Intervals for category " ++ show cat) (extractCat cat) 
+processReport opts (IntervalCategory cat) =
+    processIntervalReport opts ("Intervals for category " ++ show cat) (extractCat cat) 
     where
         extractCat :: Category -> ActivityData -> Maybe String
         extractCat cat = fmap (unpack . activityName) . listToMaybe . filter ( (==Just cat) . activityCategory )
 
-processReport opts c (IntervalTag tag) =
-    processIntervalReport opts c ("Intervals for category " ++ show tag) (extractTag tag) 
+processReport opts (IntervalTag tag) =
+    processIntervalReport opts ("Intervals for category " ++ show tag) (extractTag tag) 
     where
         extractTag :: Activity -> ActivityData -> Maybe String
         extractTag tag = fmap show . listToMaybe . filter ( (==tag) )
 
+processReport opts DumpSamples =
+    DumpResult <$> onSelected (mapElems toList $ fmap $
+        \(cd,ad) -> (tlData (cNow cd), cTimeZone cd, filterActivity (roActivityFilter opts) ad)
+        )
+
 calcCategories :: LeftFold (TimeLogEntry (Ctx, ActivityData)) [Category]
 calcCategories = fmap S.toList $ leftFold S.empty $ \s tl ->
     foldl' go' s (snd (tlData tl))
@@ -232,7 +275,7 @@ processCategoryReport opts ~(Calculations {..}) cat =
                           pick = applyActivityFilter (roActivityFilter opts) tag
                       in if pick && perc*100 >= roMinPercentage opts
                       then Just ( show tag
-                                , showTimeDiff time
+                                , showTimeDiff opts time
                                 , perc)
                       else Nothing
                       )
@@ -241,7 +284,7 @@ processCategoryReport opts ~(Calculations {..}) cat =
                 (
                 if tooSmallTimes > 0
                 then [( printf "(%d entries omitted)" (M.size tooSmallSums)
-                      , showTimeDiff tooSmallTimes
+                      , showTimeDiff opts tooSmallTimes
                       , realToFrac tooSmallTimes/realToFrac totalTimeSel
                       )]
                 else []
@@ -249,14 +292,14 @@ processCategoryReport opts ~(Calculations {..}) cat =
                 ++      
                 (if uncategorizedTime > 0
                 then [( "(unmatched time)"
-                      , showTimeDiff uncategorizedTime
+                      , showTimeDiff opts uncategorizedTime
                       , realToFrac uncategorizedTime/realToFrac totalTimeSel
                       )]
                 else []
                 )
 
-processIntervalReport :: ReportOptions -> Calculations -> String -> (ActivityData -> Maybe String) -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults
-processIntervalReport _opts _c title extr = runOnIntervals  go1 go2
+processIntervalReport :: ReportOptions -> String -> (ActivityData -> Maybe String) -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults
+processIntervalReport opts title extr = runOnIntervals  go1 go2
   where
     go1 :: LeftFold (TimeLogEntry (Ctx, ActivityData)) [Interval]
     go1 = go3 `mapElems` fmap (extr . snd) 
@@ -269,7 +312,7 @@ processIntervalReport _opts _c title extr = runOnIntervals  go1 go2
                 ( str
                 , showUtcTime (tlTime fe)
                 , showUtcTime (tlTime le)
-                , showTimeDiff $
+                , showTimeDiff opts $
                     tlTime le `diffUTCTime` tlTime fe + fromIntegral (tlRate fe)/1000
                 )
             Nothing -> Nothing) <*>
@@ -305,29 +348,35 @@ intervalReportToTable title extr = ListOfIntervals title $
 -}           
             
 renderReport :: ReportOptions -> ReportResults -> IO ()
-renderReport opts (MultpleReportResults reports) =
+renderReport opts (DumpResult samples) =
+    dumpActivity samples
+renderReport opts (MultipleReportResults reports) =
     sequence_ . intersperse (putStrLn "") . map (renderReport opts) $ reports
 renderReport opts reportdata =
     putStr $ doRender opts reportdata
 
 doRender :: ReportOptions -> ReportResults -> String
 doRender opts reportdata = case roReportFormat opts of
-                RFText -> renderReportText reportdata
-                RFCSV -> renderReportCSV reportdata
-                RFTSV -> renderReportTSV reportdata
+                RFText -> renderReportText id reportdata
+                RFCSV -> renderWithDelimiter "," $ renderXSV reportdata
+                RFTSV -> renderWithDelimiter "\t" $ renderXSV reportdata
 
-renderReportText (ListOfFields title dats) = 
-    underline title ++
+renderReportText titleMod (ListOfFields title dats) = 
+    underline (titleMod title) ++
     (tabulate False $ map (\(f,v) -> [f,v]) dats)
 
-renderReportText (ListOfTimePercValues title dats) = 
-    underline title ++ (tabulate True $ listOfValues dats)
+renderReportText titleMod (ListOfTimePercValues title dats) = 
+    underline (titleMod title) ++ (tabulate True $ listOfValues dats)
 
-renderReportText (PieChartOfTimePercValues title dats) = 
-    underline title ++ (tabulate True $ piechartOfValues dats)
+renderReportText titleMod (PieChartOfTimePercValues title dats) = 
+    underline (titleMod title) ++ (tabulate True $ piechartOfValues dats)
 
-renderReportText (ListOfIntervals title dats) = 
-    underline title ++ (tabulate True $ listOfIntervals dats)
+renderReportText titleMod (ListOfIntervals title dats) = 
+    underline (titleMod title) ++ (tabulate True $ listOfIntervals dats)
+
+renderReportText titleMod (RepeatedReportResults cat reps) = 
+    intercalate "\n" $ map (\(v,rr) -> renderReportText (titleMod . mod v) rr) reps
+  where mod v s = s ++ " (" ++ cat ++ " " ++ v ++ ")"
 
 listOfValues dats =
     ["Tag","Time","Percentage"] :
@@ -343,38 +392,24 @@ listOfIntervals dats =
 
 -- The reporting of "General Information" is not supported for the
 -- comma-separated output format.
-renderReportCSV (ListOfFields title dats) = 
-    error ("\"" ++ title ++ "\"" ++ " not supported for comma-separated output format")
+renderXSV (ListOfFields title dats) = 
+    error ("\"" ++ title ++ "\"" ++ " not supported for this output format")
 
-renderReportCSV (ListOfTimePercValues _ dats) = 
-    renderWithDelimiter "," (listOfValues dats)
+renderXSV (ListOfTimePercValues _ dats) = listOfValues dats
 
-renderReportCSV (PieChartOfTimePercValues _ dats) = 
-    renderWithDelimiter "," (piechartOfValues dats)
+renderXSV (PieChartOfTimePercValues _ dats) = piechartOfValues dats
 
-renderReportCSV (ListOfIntervals title dats) = 
-    renderWithDelimiter "," (listOfIntervals dats)
+renderXSV (ListOfIntervals title dats) = listOfIntervals dats
 
--- The reporting of "General Information" is not supported for the
--- TAB-separated output format.
-renderReportTSV (ListOfFields title dats) = 
-    error ("\"" ++ title ++ "\"" ++ " not supported for TAB-separated output format")
-
-renderReportTSV (ListOfTimePercValues _ dats) = 
-    renderWithDelimiter "\t" (listOfValues dats)
-
-renderReportTSV (PieChartOfTimePercValues _ dats) = 
-    renderWithDelimiter "\t" (piechartOfValues dats)
-
-renderReportTSV (ListOfIntervals title dats) = 
-    renderWithDelimiter "\t" (listOfIntervals dats)
+-- A bit code-smelly here.
+renderXSV (RepeatedReportResults cat reps) = title : fields
+  where
+    title = cat : head (renderXSV (snd (head reps)))
+    fields = concatMap (\(v,rr) -> map (v:) (tail (renderXSV rr))) reps
 
 renderWithDelimiter :: String -> [[String]] -> String
 renderWithDelimiter delim datasource =
-    unlines $ map (injectDelimiter delim) datasource
-
-injectDelimiter :: [a] -> [[a]] -> [a]
-injectDelimiter d = concat . intersperse d
+    unlines $ map (intercalate delim) datasource
 
 tabulate :: Bool -> [[String]] -> String
 tabulate titlerow rows = unlines $ addTitleRow $ map (intercalate " | " . zipWith (\l s -> take (l - length s) (repeat ' ') ++ s) colwidths) rows
@@ -385,8 +420,12 @@ tabulate titlerow rows = unlines $ addTitleRow $ map (intercalate " | " . zipWit
                  -- | titlerow  = \(l:ls) -> l : (take (length l) (repeat '-')) : ls
                     | otherwise = id
 
-showTimeDiff :: NominalDiffTime -> String
-showTimeDiff t = go False $ zip [days,hours,mins,secs] ["d","h","m","s"]
+showTimeDiff :: ReportOptions -> NominalDiffTime -> String
+showTimeDiff (ReportOptions { roReportFormat = RFText }) = showTimeDiffHuman
+showTimeDiff _                                           = showTimeDiffMachine
+
+showTimeDiffHuman :: NominalDiffTime -> String
+showTimeDiffHuman t = go False $ zip [days,hours,mins,secs] ["d","h","m","s"]
   where s = round t :: Integer
         days  =  s `div` (24*60*60)
         hours = (s `div` (60*60)) `mod` 24
@@ -399,6 +438,13 @@ showTimeDiff t = go False $ zip [days,hours,mins,secs] ["d","h","m","s"]
         go False ((a,u):vs) | a > 0     = printf "%2d%s" a u ++ go True vs
                             | otherwise =                       go False vs
 
+showTimeDiffMachine :: NominalDiffTime -> String
+showTimeDiffMachine t = printf "%d:%02d:%02d" hours mins secs
+  where s = round t :: Integer
+        hours = s `div` (60*60)
+        mins  = (s `div` 60) `mod` 60
+        secs  =  s `mod` 60 
+
 showUtcTime :: UTCTime -> String
 showUtcTime = formatTime defaultTimeLocale "%x %X"
 
