@@ -6,7 +6,7 @@ import System.Environment
 import System.Exit
 import System.IO
 import Control.Monad
-import qualified Data.MyText as T
+import Data.Maybe
 import Data.Char (toLower)
 import Text.Printf
 import Data.Version (showVersion)
@@ -17,18 +17,21 @@ import Data.ByteString.Lazy.Progress
 import System.Posix.Files
 import System.ProgressBar
 import TermSize
+import qualified Data.MyText as T
 
 import TimeLog
 import Categorize
 import Stats
 import CommonStartup
 import LeftFold
+import DumpFormat
 
 import Paths_arbtt (version)
 
 data Options = Options
     { optReports :: [Report]
     , optFilters :: [Filter]
+    , optRepeater :: [Repeater]
     , optAlsoInactive :: Bool
     , optReportOptions :: ReportOptions
     , optLogFile :: String
@@ -39,6 +42,7 @@ defaultOptions :: FilePath -> Options
 defaultOptions dir = Options
     { optReports = []
     , optFilters = []
+    , optRepeater = []
     , optAlsoInactive = False
     , optReportOptions = defaultReportOptions
     , optLogFile = dir </> "capture.log"
@@ -120,21 +124,35 @@ options =
                                        reports = report : optReports opt
                                    in  return opt { optReports = reports }) "TAG")
               "list intervals of tag or category TAG"
+     , Option ""       ["dump-samples"]
+              (NoArg (\opt ->      let reports = DumpSamples : optReports opt
+                                   in  return opt { optReports = reports }))
+              "Dump the raw samples and tags."
      , Option ""       ["output-format"]
               (ReqArg (\arg opt -> let ro = (optReportOptions opt) { roReportFormat = readReportFormat arg }
                                    in  return opt { optReportOptions = ro }) "FORMAT")
               "one of: text, csv (comma-separated values), tsv (TAB-separated values) (default: Text)"
+     , Option ""       ["for-each"]
+              (ReqArg (\arg opt -> let repeater = readRepeater arg : optRepeater opt
+                                   in  return opt { optRepeater = repeater }) "PERIOD")
+              "one of: day, month, year"
      ]
 
+readRepeater :: String -> Repeater
+readRepeater arg =
+    case map toLower arg of
+        "day"   -> ByDay
+        "month" -> ByMonth
+        "year"  -> ByYear
+        _       -> error ("Unsupported parameter to --for-each: '" ++ arg ++ "'")
+
 readReportFormat :: String -> ReportFormat
 readReportFormat arg =
-    case (tolower arg) of
+    case map toLower arg of
         "text" -> RFText
         "csv"  -> RFCSV
         "tsv"  -> RFTSV
         _      -> error ("Unsupported report output format: '" ++ arg ++ "'")
-    where
-        tolower = map toLower
 
 main :: IO ()
 main = do
@@ -158,18 +176,22 @@ main = do
 
   timelog <- BS.readFile (optLogFile flags)
   size <- fileSize <$> getFileStatus (optLogFile flags)
+  isTerm <- hIsTerminalDevice stderr
 
-  hSetBuffering stderr NoBuffering
-  trackedTimelog <- trackProgressWithChunkSize (fromIntegral size `div` 100) (\_ b -> do
-    (_height, width) <- getTermSize
-    hPutChar stderr '\r'
-    hPutStr stderr $
-        mkProgressBar (msg "Processing data") percentage (fromIntegral width) (fromIntegral b) (fromIntegral size)
-    when  (fromIntegral b >= fromIntegral size) $ do
+  trackedTimelog <- case isTerm of
+    True -> do
+      hSetBuffering stderr NoBuffering
+      trackProgressWithChunkSize (fromIntegral size `div` 100) (\_ b -> do
+        (_height, width) <- getTermSize
         hPutChar stderr '\r'
-        hPutStr stderr (replicate width ' ')
-        hPutChar stderr '\r'
-    ) timelog
+        hPutStr stderr $
+            mkProgressBar (msg "Processing data") percentage (fromIntegral width) (fromIntegral b) (fromIntegral size)
+        when  (fromIntegral b >= fromIntegral size) $ do
+            hPutChar stderr '\r'
+            hPutStr stderr (replicate width ' ')
+            hPutChar stderr '\r'
+        ) timelog
+    False -> return timelog
 
   let captures = parseTimeLog trackedTimelog
   let allTags = categorizer captures
@@ -178,19 +200,24 @@ main = do
      exitFailure
       
   let filters = (if optAlsoInactive flags then id else (defaultFilter:)) $ optFilters flags
-  let reps = case optReports flags of {[] -> [TotalTime]; reps -> reverse reps }
+
+  let rep = case optReports flags of
+                [] -> TotalTime
+                [x] -> x
+                _ -> error "Please specify exactly one report to generate"
+  let repeater = foldr (.) id $ map processRepeater (optRepeater flags)
 
   -- These are defined here, but of course only evaluated when any report
   -- refers to them. Some are needed by more than one report, which is then
   -- advantageous.
   let opts = optReportOptions flags
-  let (c,results) = runLeftFold (filterPredicate filters `filterWith` 
-        (pure (,) <*> prepareCalculations <*> processReports opts c reps)) allTags
+  let fold = filterPredicate filters `adjoin` repeater (processReport opts rep)
+  let result = runLeftFold fold allTags
 
-  -- Force the results a bit, to ensure the progress bar to be shown before the titel
-  c `seq` return ()
+  -- Force the results a bit, to ensure the progress bar to be shown before the title
+  result `seq` return ()
   
-  renderReport opts (MultpleReportResults results)
+  renderReport opts result
 
 {-
 import Data.Accessor
